/* -*- mode: C; mode: fold; -*- */
/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <string.h>

#include "jdmath.h"
#include "jdsys.h"

#ifdef NEEDS_BYTE_SWAP
static void byte_swap64 (unsigned char *ss, unsigned int n) /*{{{*/
{
   unsigned char *p, *pmax, ch;
   
   if (n <= 0) return;
   p = (unsigned char *) ss;
   pmax = p + 8 * n;
   while (p < pmax)
     {
	ch = *p;
	*p = *(p + 7);
	*(p + 7) = ch;
	
	ch = *(p + 6);
	*(p + 6) = *(p + 1);
	*(p + 1) = ch;
	
	ch = *(p + 5);
	*(p + 5) = *(p + 2);
	*(p + 2) = ch;
	
	ch = *(p + 4);
	*(p + 4) = *(p + 3);
	*(p + 3) = ch;
	
	p += 8;
     }
}

/*}}}*/
static void byte_swap32 (unsigned char *ss, unsigned int n) /*{{{*/
{
   unsigned char *p, *pmax, ch;
   
   p = (unsigned char *) ss;
   pmax = p + 4 * n;
   while (p < pmax)
     {
	ch = *p;
	*p = *(p + 3);
	*(p + 3) = ch;
	
	ch = *(p + 1);
	*(p + 1) = *(p + 2);
	*(p + 2) = ch;
	p += 4;
     }
}

/*}}}*/
static void byte_swap16 (unsigned char *p, unsigned int nread) /*{{{*/
{
   unsigned char *pmax, ch;
   
   pmax = p + 2 * nread;
   while (p < pmax)
     {
	ch = *p;
	*p = *(p + 1);
	*(p + 1) = ch;
	p += 2;
     }
}

/*}}}*/
#endif

/*{{{ File Read Functions */

unsigned int JDMread_int32 (int32 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nread = fread (ss, 4, n, fp);
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) ss, nread);
#endif
   return nread;
}

/*}}}*/

unsigned int JDMread_int16 (int16 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nread = fread (ss, 2, n, fp);
#ifdef NEEDS_BYTE_SWAP
   byte_swap16 ((unsigned char *) ss, nread);
#endif
   return nread;
}

/*}}}*/

unsigned int JDMread_float64 (float64 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nread = fread (ss, 8, n, fp);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap64 ((unsigned char *)ss, nread);
#endif
   return nread;
}

/*}}}*/

unsigned int JDMread_float32 (float32 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nread = fread (ss, 4, n, fp);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *)ss, nread);
#endif
   return nread;
}

/*}}}*/

unsigned int JDMread_f_float32 (float *f, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_FLOAT == 4
   return JDMread_float32 ((float32 *) f, n, fp);
#else
   /* Do it the hard way. */
   float32 f32;
   unsigned int i;
   
   for (i = 0; i < n; i++)
     {
	if (1 != JDMread_float32 (&f32, 1, fp))
	  break;
	
	f[i] = (float) f32;
     }
   return i;
#endif
}

/*}}}*/

unsigned int JDMread_d_float64 (double *d, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_DOUBLE == 8
   return JDMread_float64 ((float64 *) d, n, fp);
#else
   float64 f64;
   unsigned int i;
   
   for (i = 0; i < n; i++)
     {
	if (1 != JDMread_float64 (&f64, 1, fp))
	  break;
	
	d[i] = (double) f64;
     }
   return i;
#endif
}

/*}}}*/

unsigned int JDMread_d_float32 (double *d, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_DOUBLE == 4
   /* Pity the person who has this system */
   return JDMread_float32 ((float32 *) d, n, fp);
#else
   /* Do it the hard way. */
   float32 f32;
   unsigned int i;
   
   for (i = 0; i < n; i++)
     {
	if (1 != JDMread_float32 (&f32, 1, fp))
	  break;
	
	d[i] = (double) f32;
     }
   
   return i;
#endif
}

/*}}}*/

unsigned int JDMread_f_float64 (float *f, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_FLOAT == 8
   return JDMread_float64 ((float64 *) f, n, fp);
#else
   float64 f64;
   unsigned int i;
   
   for (i = 0; i < n; i++)
     {
	if (1 != JDMread_float64 (&f64, 1, fp))
	  break;
	
	f[i] = (float) f64;
     }
   return i;
#endif
}

/*}}}*/

/*}}}*/

/*{{{ File Write Functions */

unsigned int JDMwrite_float32 (float32 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nwrote;
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) ss, n);
#endif
   nwrote = fwrite (ss, 4, n, fp);
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) ss, n);
#endif
   return nwrote;
}

/*}}}*/

unsigned int JDMwrite_int32 (int32 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nwrote;
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) ss, n);
#endif
   nwrote = fwrite (ss, 4, n, fp);
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) ss, n);
#endif
   return nwrote;
}

/*}}}*/

unsigned int JDMwrite_int16 (int16 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nwrote;
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap16 ((unsigned char *) ss, n);
#endif
   nwrote = fwrite (ss, 2, n, fp);
#ifdef NEEDS_BYTE_SWAP
   byte_swap16 ((unsigned char *) ss, n);
#endif
   return nwrote;
}

/*}}}*/

unsigned int JDMwrite_float64 (float64 *ss, unsigned int n, FILE *fp) /*{{{*/
{
   unsigned int nwrote;
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap64 ((unsigned char *) ss, n);
#endif
   nwrote = fwrite (ss, 8, n, fp);
#ifdef NEEDS_BYTE_SWAP
   byte_swap64 ((unsigned char *) ss, n);
#endif
   return nwrote;
}

/*}}}*/

unsigned int JDMwrite_f_float32 (float *f, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_FLOAT == 4
   return JDMwrite_float32 ((float32 *) f, n, fp);
#else
   float32 f32;
   unsigned int i;
   
   for (i = 0; i < n; i++) 
     {
	f32 = (float32) f[i];
	if (1 != JDMwrite_float32 (&f32, 1, fp))
	  break;
     }
   return i;
#endif
}

/*}}}*/

unsigned int JDMwrite_d_float32 (double *d, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_DOUBLE == 4
   return JDMwrite_float32 ((float32 *) d, n, fp);
#else
   float32 f32;
   unsigned int i;
   
   for (i = 0; i < n; i++) 
     {
	f32 = (float32) d[i];
	if (1 != JDMwrite_float32 (&f32, 1, fp))
	  break;
     }
   return i;
#endif
}

/*}}}*/

unsigned int JDMwrite_f_float64 (float *f, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_FLOAT == 8
   return JDMwrite_float64 ((float64 *) f, n, fp);
#else
   float64 f64;
   unsigned int i;
   
   for (i = 0; i < n; i++) 
     {
	f64 = (float64) f[i];
	if (1 != JDMwrite_float64 (&f64, 1, fp))
	  break;
     }
   return i;
#endif
}

/*}}}*/

unsigned int JDMwrite_d_float64 (double *d, unsigned int n, FILE *fp) /*{{{*/
{
#if SIZEOF_DOUBLE == 8
   return JDMwrite_float64 ((float64 *) d, n, fp);
#else
   float64 f64;
   unsigned int i;
   
   for (i = 0; i < n; i++) 
     {
	f64 = (float64) d[i];
	if (1 != JDMwrite_float64 (&f64, 1, fp))
	  break;
     }
   return i;
#endif
}

/*}}}*/

/*}}}*/

/*{{{ Reading from a string Functions */

unsigned char *JDMstr_read_int32 (int32 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 4 * n;
   if (s != (unsigned char *) ss) memcpy ((char *) ss, (char *) s, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) ss, n);
#endif
   return s + len;
}

/*}}}*/

unsigned char *JDMstr_read_int16 (int16 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 2 * n;
   
   if (s != (unsigned char *)ss) memcpy ((char *) ss, (char *) s, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap16 ((unsigned char *) ss, n);
#endif
   return s + len;
}

/*}}}*/

unsigned char *JDMstr_read_float64 (float64 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 8 * n;
   
   if (s != (unsigned char *)ss) memcpy ((char *) ss, (char *) s, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap64 ((unsigned char *)ss, n);
#endif
   return s + len;
}

/*}}}*/

unsigned char *JDMstr_read_float32 (float32 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 4 * n;
   if (s != (unsigned char *)ss) memcpy ((char *) ss, (char *) s, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *)ss, n);
#endif
   return s + len;
}

/*}}}*/


/*}}}*/

/*{{{ Writing to a string Functions */

unsigned char *JDMstr_write_int32 (int32 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 4 * n;
   
   memcpy ((char *) s, (char *) ss, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) s, n);
#endif
   return s + len;
}

/*}}}*/

unsigned char *JDMstr_write_int16 (int16 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 2 * n;
   
   memcpy ((char *) s, (char *) ss, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap16 ((unsigned char *) s, n);
#endif
   return s + len;
}

/*}}}*/

unsigned char *JDMstr_write_float32 (float32 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 4 * n;
   
   memcpy ((char *) s, (char *) ss, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap32 ((unsigned char *) s, n);
#endif
   return s + len;
}

/*}}}*/

unsigned char *JDMstr_write_float64 (float64 *ss, unsigned int n, unsigned char *s) /*{{{*/
{
   unsigned int len = 8 * n;
   
   memcpy ((char *) s, (char *) ss, len);
   
#ifdef NEEDS_BYTE_SWAP
   byte_swap64 ((unsigned char *) s, n);
#endif
   return s + len;
}

/*}}}*/

/*}}}*/

