/* Copyright (c) 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <string.h>

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#include "jdmath.h"
#include "_jdmath.h"

#define TINY_NUMBER 1.0e-20

/* This routine was adapted from numerical recipes ludcmp.  
 * dptr is allowed to be NULL.
 */
int JDM_lu_decomp (double **a, unsigned int n, unsigned int *indx, int *dptr)
{
   double *vv;
   int d;
   unsigned int i, j, imax;
   double largest;

   if ((n == 0) || (a == NULL) || (indx == NULL))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }

   if (NULL == (vv = JDMdouble_vector (n)))
     return -1;
   
   d = 1;

   for (i = 0; i < n; i++)
     {
	double *ai;
	
	largest = 0.0;
	ai = a[i];
	for (j = 0; j < n; j++) 
	  {
	     double aij = fabs (ai[j]);
	     if (aij > largest) largest = aij;
	  }
	
	if (largest == 0.0)
	  {
	     JDMmsg_error ("JDM_lu_decomp: singular matrix");
	     JDMfree_double_vector (vv);
	     return -1;
	  }
	vv[i] = 1.0 / largest;
     }
   
   imax = 0;
   for (j = 0; j < n; j++)
     {
	double sum;
	double *ai;

	for (i = 0; i < j; i++)
	  {
	     unsigned int k;
	     
	     ai = a[i];
	     sum = ai[j];
	     
	     for (k = 0; k < i; k++) sum -= ai[k] * a[k][j];
	     ai[j] = sum;
	  }
		  
	largest = 0.0;
	
	for (i = j; i < n; i++)
	  {
	     unsigned int k;

	     ai = a[i];
	     sum = ai[j];
	     
	     for (k = 0; k < j; k++) sum -= ai[k] * a[k][j];
	     ai[j] = sum;
	
	     sum = vv[i] * fabs(sum);
	     if (sum >= largest)
	       {
		  largest = sum;
		  imax = i;
	       }
	  }
	
	if (j != imax)
	  {
	     _JDMswap_dvector (a[imax], a[j], n);
	     d = -d;
	     vv[imax] = vv[j];
	  }
	
	indx[j] =imax;

	if (0 == (sum = a[j][j])) 
	  sum = a[j][j] = TINY_NUMBER;
	
	sum = 1.0/sum;
	for (i = j + 1; i < n; i++)
	  a[i][j] = a[i][j] * sum;
     }
	     
   JDMfree_double_vector (vv);
   if (dptr != NULL)
     *dptr = d;
   
   return 0;
}

int 
JDM_lu_backsubst (double **a, unsigned int n, unsigned int *indx, double *b)
{
   double sum;
   unsigned int i;
   int ii;

   if ((a == NULL) || (b == NULL) || (indx == NULL) || (n == 0))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }
   
   ii = -1;
   for (i = 0; i < n; i++)
     {
	unsigned int k, j;
	double *ai;

	k = indx[i];
	sum = b[k];
	b[k] = b[i];
	
	ai = a[i];
	if (ii != -1) 
	  for (j = (unsigned int) ii; j < i; j++) sum -= ai[j] * b[j];
	else if (sum != 0)
	  ii = i;
	
	b[i] = sum;
     }
   
   i = n;
   while (i)
     {
	unsigned int j;
	double *ai;

	j = i;

	i--;
	sum = b[i];
	ai = a[i];

	while (j < n)
	  {
	     sum -= ai[j] * b[j];
	     j++;
	  }
	/* Note!! ai[i] could be zero! See how TINY_NUMBER is introduced above */
	b[i] = sum / ai[i];
     }
   
   return 0;
}

int JDM_ludecomp_inverse (double **a, unsigned int n)
{
   unsigned int *indx;
   double *col;
   double **inv_a;
   unsigned int i, j;
   int ret;

   if (n == 0)
     return 0;

   if (a == NULL)
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }

   indx = NULL;
   col = NULL;
   inv_a = NULL;
   ret = -1;

   if ((NULL == (indx = (unsigned int *)JDMinteger_vector (n)))
       || (NULL == (col = JDMdouble_vector (n)))
       || (NULL == (inv_a = JDMdouble_matrix (n, n))))
     goto return_error;

   if (-1 == JDM_lu_decomp (a, n, indx, NULL))
     goto return_error;
   
   for (j = 0; j < n; j++)
     {
	for (i = 0; i < n; i++) col[i] = 0;
	col[j] = 1;

	if (-1 == JDM_lu_backsubst (a, n, indx, col))
	  goto return_error;
	
	for (i = 0; i < n; i++)
	  inv_a [i][j] = col[i];
     }
   
   for (i = 0; i < n; i++)
     {
	double *ai, *inv_ai;
	
	ai = a[i];
	inv_ai = inv_a[i];
	for (j = 0; j < n; j++)
	  ai[j] = inv_ai[j];
     }
   
   ret = 0;
   /* drop */

   return_error:
   JDMfree_integer_vector ((int *) indx);
   JDMfree_double_vector (col);
   JDMfree_double_matrix (inv_a, n);
   
   return ret;
}

#ifdef TEST_LUDECOMP
#include <stdio.h>
#include "jdmath.h"

int main (int argc, char **argv)
{
   unsigned int n;
   unsigned int i, j;
   double **a, **a1;
   unsigned int count;

   n = 20;
   
   a = JDMdouble_matrix (n, n);
   a1 = JDMdouble_matrix (n, n);

   
   count = 20;
   
   while (count)
     {
	double max;

	count--;

	for (i = 0; i < n; i++)
	  {
	     for (j = 0; j < n; j++)
	       a[i][j] = a1[i][j] = JDMrandom ();
	  }
   
	if (-1 == JDM_ludecomp_inverse (a, n))
	  continue;
	
	max = 0;

	for (i = 0; i < n; i++)
	  {
	     unsigned int k;
	     for (k = 0; k < n; k++)
	       {
		  double sum = 0;
		  for (j = 0; j < n; j++)
		    sum += a[i][j] * a1[j][k];
		  
		  if (i == k)
		    sum -= 1.0;
		  
		  if (fabs (sum) > max) max = fabs(sum);
	       }
	  }
	fprintf (stdout, "%e\n", max);
     }

   return 0;
}

#endif
	
	
	
	
   
