/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <math.h>


#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif
#include <string.h>

#include "jdmath.h"

JDMVector_Type JDMv_vector (double x, double y, double z)
{
   JDMVector_Type a;
   a.x = x;
   a.y = y;
   a.z = z;
   return a;
}

JDMVector_Type JDMv_cross_prod (JDMVector_Type a, JDMVector_Type b)
{
   JDMVector_Type c;
   c.z = a.x * b.y - a.y * b.x;
   c.x = a.y * b.z - a.z * b.y;
   c.y = a.z * b.x - a.x * b.z;
   
   return c;
}

JDMVector_Type JDMv_pcross_prod (JDMVector_Type *a, JDMVector_Type *b)
{
   JDMVector_Type c;
   c.z = a->x * b->y - a->y * b->x;
   c.x = a->y * b->z - a->z * b->y;
   c.y = a->z * b->x - a->x * b->z;
   
   return c;
}

JDMVector_Type JDMv_smul (double x, JDMVector_Type a)
{
   a.x *= x;
   a.y *= x;
   a.z *= x;
   return a;
}

double JDMv_dot_prod (JDMVector_Type a, JDMVector_Type b)
{
   return a.x * b.x + a.y * b.y + a.z * b.z;
}

double JDMv_pdot_prod (JDMVector_Type *a, JDMVector_Type *b)
{
   return a->x * b->x + a->y * b->y + a->z * b->z;
}

double JDMv_length (JDMVector_Type a)
{
   double x, y, z, tmp;
   
   x = fabs (a.x);
   y = fabs (a.y);
   z = fabs (a.z);
   
   if (z < x)
     {
	tmp = z; z = x; x = tmp;
     }
   if (z < y)
     {
	tmp = z; z = y; y = tmp;
     }
   if (z == 0.0) return 0.0;
   x = x / z; y = y / z;
   z = z * sqrt (1.0 + x * x + y * y);
   return z;
}

extern void JDMv_normalize (JDMVector_Type *a)
{
   double len;
   
   if ((len = JDMv_length (*a)) != 0.0)
     {
	a->x = a->x / len;
	a->y = a->y / len;
	a->z = a->z / len;
     }
}


JDMVector_Type JDMv_unit_vector (JDMVector_Type a)
{
   JDMVector_Type n;
   
   n = a;
   JDMv_normalize (&n);
   return n;
}


JDMVector_Type JDMv_ax1_bx2 (double a, JDMVector_Type x1, 
			     double b, JDMVector_Type x2)
{
   JDMVector_Type c;
   
   c.x = a * x1.x + b * x2.x;
   c.y = a * x1.y + b * x2.y;
   c.z = a * x1.z + b * x2.z;
   
   return c;
}

JDMVector_Type JDMv_pax1_bx2 (double a, JDMVector_Type *x1,
			      double b, JDMVector_Type *x2)
{
   JDMVector_Type c;
   
   c.x = a * x1->x + b * x2->x;
   c.y = a * x1->y + b * x2->y;
   c.z = a * x1->z + b * x2->z;
   
   return c;
}

JDMVector_Type JDMv_sum (JDMVector_Type x1, JDMVector_Type x2)
{
   JDMVector_Type c;
   
   c.x = x1.x + x2.x;
   c.y = x1.y + x2.y;
   c.z = x1.z + x2.z;
   
   return c;
}

JDMVector_Type JDMv_diff (JDMVector_Type x1, JDMVector_Type x2)
{
   JDMVector_Type c;
   
   c.x = x1.x - x2.x;
   c.y = x1.y - x2.y;
   c.z = x1.z - x2.z;
   
   return c;
}

double JDMv_distance (JDMVector_Type a, JDMVector_Type b)
{
   a.x -= b.x;
   a.y -= b.y;
   a.z -= b.z;
   
   return JDMv_length (a);
}

JDMVector_Type JDMv_rotate_unit_vector (JDMVector_Type p, JDMVector_Type n, 
					double theta)
{
   JDMVector_Type pn;
   
   pn = JDMv_smul (JDMv_dot_prod (p, n), n);
   return JDMv_sum (pn,
		    JDMv_ax1_bx2 (cos(theta), JDMv_diff(p, pn),
				  sin(theta), JDMv_pcross_prod(&n,&p)));
}


